from numpy import *

## Class to build and access a dictionary mapping between string names
## and numbers.  Used to assign a unique number to each variable.
## For example:
## >>n2n = name2num()
## >>n2n.insert('n1')
## >>n2n.insert('n2')
## >>n2n.insert('n1')   # has no effect since it is a duplicate
## >>n2n.lookup('n1')
## 0
## >>n2n.names()
## ['n1', 'n2']

## Maintains a mapping from inserted names to contiguous integers,
## starting with 0
class name2num:
    def __init__(self):
        self.next_int = 0
        self.names2nums = {}
    def insert(self, x):
        # Assigns node number to a name that hasn't been inserted
        # before; inserting a name subsequent times has no effect
        if not self.names2nums.has_key(x):
            self.names2nums[x] = self.next_int
            self.next_int = self.next_int + 1
    def lookup(self, x):
        # Return the node number for name x.  Error if x hasn't been
        # inserted 
        return self.names2nums[x]
    def names(self):
        # Return list of variable names
        return self.names2nums.keys()
        
## Maintains a list of equations and a dictionary of names to numbers.
## The dictionary is used when we convert equations to a matrix of
## coefficients, to map each variable name to a column of the matrix.    
## Has methods to solve the equations and support printing the
## solutions.

class ConstraintSet:
    def __init__(self):
        self.equations = []             # list of equations
        self.n2n = name2num()           # var name -> number mapping

    # eqn is a list of (number string) or (number None) expressing a
    # constraint
    def addConstraint(self, eqn):
        # Add eqn to this set of constraints and add all of the
        # variable names from this equation into the self.n2n

        ## Your code here.
        pass
        
    def listVariables(self):
        #  Return a list of variable names without duplicates
        return self.n2n.names()

    def solve(self):
        # returns vector of variable values satisifying the
        # constraints, by solving linear equations.
        vars = len(self.listVariables())
        eqs = len(self.equations)
        if not(vars == eqs):
            print 'Number of variables = ', vars
            print 'Does not match number of eqns = ', eqs
            print 'Did you forget a conservation law or to set a ground?'
            assert vars == eqs, 'equation/variable mismatch error'
        c = array([0.0]*eqs)
        A = array([[0.0 for col in range(vars)] for row in range(eqs)])
        # Write the coefficients into the A matrix and the constants
        # in the c vector.  Return a vector of values for the variables.

        ## Your code here.
        pass

    # solution is a vector of values, variable is a string name
    # returns the value of that variable in the solution
    def translate(self, variable, solution):
        return solution[self.n2n.lookup(variable)]

    # solution is a vector of values
    # prints each variable name and its associated value in the solution
    def display(self, solution):
        varlist = self.listVariables()
        varlist.sort()                  # alphabetical order
        for var in varlist:
            print var, ' = ', self.translate(var, solution)

